@@ -1,7 +1,6 @@
 import logging
 import os
 import sys
-import threading
 import warnings
 from functools import partial
 from multiprocessing import Process
@@ -18,6 +17,8 @@ from pyqalx.core.session import QalxSession
 from pyqalx.core.errors import (
     QalxBotInitialisationFailed,
     QalxStepFunctionNotDefined,
+    QalxMultipleEntityReturned,
+    QalxEntityNotFound,
 )
 from pyqalx.core.errors import QalxError, QalxConfigError
 from pyqalx.core.log import (
@@ -26,6 +27,7 @@ from pyqalx.core.log import (
     QalxLogFormatter,
 )
 from pyqalx.core.signals import QalxSignal
+from pyqalx.factories.factory import Factory
 
 logger = logging.getLogger(__name__)
 
@@ -78,6 +80,7 @@ class QalxJob:
         "_bot_entity",
         "_queue_message",
         "_step_results",
+        "do_workflow",
     )
 
     def __init__(self, worker_process):
@@ -90,6 +93,7 @@ class QalxJob:
 
         self._current_step = "class init"
         self._step_results = []
+        self.do_workflow = True
 
     @property
     def log(self):
@@ -322,6 +326,7 @@ class Bot:
         "user_config_class",
         "bot_config_class",
         "entity_classes",
+        "workflow",
     )
 
     def __init__(self, bot_name):
@@ -340,7 +345,7 @@ class Bot:
 
         # Use `partial` so we can ensure the name of the function always goes
         # to the exception.
-        # TODO:Figure out a nicer way to get the step_name in here
+        # TODO:#3927 Figure out a nicer way to get the step_name in here
         self.process_function = partial(
             cannot_be_undefined, step_name="process"
         )
@@ -376,6 +381,9 @@ class Bot:
         bot_profile_name="default",
         skip_ini=False,
         add_queue=True,
+        stop=False,
+        meta=None,
+        workflow=None,
     ):
         """
         :param queue_name: The unique name for the queue
@@ -405,9 +413,27 @@ class Bot:
         :param add_queue: Should the queue be created if it doesn't already
                           exist
         :type add_queue: bool
+        :param stop:    Should the bot be stopped right after startup. This is
+                        essential functionality during factory building
+        :type  stop:    bool
+        :param meta:        Metadata to be saved on the bot. Optional parameter
+        :type  meta:        dict
+        :param workflow:    An iterable of other bots that defines the
+                            interaction of the bot with other bots in the
+                            context of a workflow
+        :type  workflow:    list
         """
 
         self.queue_name = queue_name
+        # This is needed for starting outside the CLI
+        workflow = workflow or []
+        if not isinstance(workflow, list) or not all(
+            [isinstance(_, str) for _ in workflow]
+        ):
+            raise QalxError(
+                "Workflow argument must be an iterable containing strings"
+            )
+        self.workflow = workflow
 
         if entity_classes is None:
             entity_classes = []
@@ -455,7 +481,7 @@ class Bot:
         # specific queues.
         bot_session.config.update({"queue_name": self.queue_name})
         self.bot_entity = user_session.bot.add(
-            name=self.name, config=bot_session.config
+            name=self.name, config=bot_session.config, meta=meta
         )
 
         self.bot_adapter = bot_session.bot
@@ -479,7 +505,8 @@ class Bot:
         self.bot_entity = self.bot_adapter.replace_workers(
             bot_entity=self.bot_entity, workers=processes
         )
-
+        if stop is True:
+            self.bot_adapter.stop(self.bot_entity)
         workers = []
         # We use the same session and adapter for all workers
 
@@ -494,6 +521,7 @@ class Bot:
                     self.bot_entity["workers"][worker],
                     worker_adapter,
                     index=worker,
+                    workflow=self.workflow,
                 )
             )
             logger.debug(
@@ -613,6 +641,7 @@ class Worker(Process):
         api_entity,
         worker_adapter,
         index,
+        workflow,
         **kwargs,
     ):
         super(Worker, self).__init__(**kwargs)
@@ -622,10 +651,13 @@ class Worker(Process):
         self.api_entity = api_entity
         self.worker_adapter = worker_adapter
         self.session = self.worker_adapter.session
+
+        self.logger = None
         self.queue_adapter = QalxQueue(session=self.session)
         self.stop = False
         self.config = self.session.config
         self.index = index
+        self.workflow = workflow
         self.worker_signal = None
 
     def configure_logging(self, index):
@@ -716,13 +748,58 @@ class Worker(Process):
             # deleted.
             self._do_exit(**QalxSignal._warm_exit_update())
 
+    def _workflow(self):
+        """
+        Workflow step function. Used to handle passing a job to multiple bots
+        once all step functions have been completed on the worker
+        """
+        self.logger.debug("Attempting worklow")
+        factory_namespace_key = Factory.PYQALX_METADATA_NAMESPACE_KEY
+        factory_guid_key = Factory.PYQALX_METADATA_FACTORY_GUID_KEY
+        factory_plan_bot_name_key = Factory.PYQALX_METADATA_PLAN_BOT_NAME_KEY
+        factory_guid = self.bot_entity.meta.get(factory_namespace_key, {}).get(
+            factory_guid_key, None
+        )
+        for bot_name in self.workflow:
+            self.logger.debug(
+                f"Workflow found." f"Next bot in workflow: `{bot_name}`"
+            )
+            # By default, just try and find the bot based on the name
+            query = {"name": bot_name}
+            if factory_guid is not None:
+                # If it's a factory bot then we need to use the plan_bot_name
+                # and the factory_guid, as the actual bot_name won't be known
+                # when the factory was built
+                pyqalx_query = f"metadata.data.{factory_namespace_key}"
+                query = {
+                    "metadata.data._class": "pyqalx.factory",
+                    f"{pyqalx_query}.{factory_plan_bot_name_key}": bot_name,
+                    f"{pyqalx_query}.{factory_guid_key}": factory_guid,
+                }
+            try:
+                bot = self.session.bot.find_one(query=query)
+            except (QalxMultipleEntityReturned, QalxEntityNotFound) as exc:
+                warn_msg = (
+                    f"There was an error retrieving one bot "
+                    f"for `{bot_name}` in the defined workflow. "
+                    f"Query was `{query}`: `{exc}`"
+                )
+                self.logger.warning(warn_msg)
+                continue
+            queue = self.session.queue.get_by_name(name=bot.config.queue_name)
+            self.logger.debug(
+                f"Submitting `{self.job.entity}` "
+                f"to `{queue.name}` on workflow bot"
+                f" name `{bot_name}`"
+            )
+            queue.submit_entity(self.job.entity)
+
     def _delete_message(self):
         """
         Helper function for stopping any heartbeat on the job and
         deleting the message
         """
         if getattr(self.job, "_queue_message", None):
-            self.job._queue_message._stop_heartbeat()
             self.queue.delete_message(self.job._queue_message)
 
     def _wait_with_spreading(self, spread_factor):
@@ -781,7 +858,7 @@ class Worker(Process):
             except TypeError:
                 self.steps[f_name] = f_dill
 
-        self.logger.debug(f"running")
+        self.logger.debug("running")
         if self.steps.get("begin_function"):
             self.job._current_step = "Begin"
 
@@ -804,7 +881,7 @@ class Worker(Process):
                     f"Stop signal on Worker waited with factor {spread_factor}"
                 )
             else:
-                q_msgs = self.queue_adapter.get_messages(queue=self.queue)
+                q_msgs = self.queue_adapter.get_messages(worker=self)
                 self.logger.debug(f"got {len(q_msgs)} messages")
                 if q_msgs:
                     for q_msg in q_msgs:
@@ -858,7 +935,9 @@ class Worker(Process):
                     break
             if not self.config.getboolean("SKIP_FINAL_SAVE"):
                 self.job.save_entity()
-
+            # `_workflow` is executed only if the flag on the job entity is True
+            if self.job.do_workflow is True:
+                self._workflow()
             # We delete the message from the queue once we've processed it
             self._delete_message()
             return self.job
@@ -887,19 +966,7 @@ class Worker(Process):
             entity_guid = self.job._queue_message.body["entity_guid"]
             self.job.get_entity(entity_type, entity_guid)
 
-            t = threading.Thread(
-                target=process_as_far_as_possible, args=(process_functions,)
-            )
-            t.start()
-            while t.is_alive():
-                # Start a heartbeat while we process this message.
-                visibility = self.config["MSG_BLACKOUTSECONDS"]
-                self.job._queue_message._do_heartbeat(
-                    visibility=visibility, queue=self.queue
-                )
-                # Ensure the credentials are still valid after the heartbeat.
-                # Especially important for long running heartbeats.
-                self._check_queue_credentials()
+            process_as_far_as_possible(process_functions)
         else:
             self._delete_message()
 
