@@ -39,32 +39,43 @@ class Config(dict):
     def config_path(cls):
         return os.path.join(os.path.expanduser("~"), cls.filename)
 
-    def from_inifile(self, profile_name="default"):
-
-        if os.path.exists(self.config_path()):
+    @classmethod
+    def _load_inifile(cls, profile_name):
+        """
+        Loads the inifile and returns the values of the specific
+        profile_name as a dict
+        :param profile_name: The name of the profile to load
+        :return: Dict of config options
+        :raises: QalxConfigProfileNotFound
+        """
+        if os.path.exists(cls.config_path()):
             config = configparser.ConfigParser()
             config.optionxform = str
-            with open(self.config_path()) as cfg:
+            with open(cls.config_path()) as cfg:
                 config.read_string(
                     cfg.read()
                 )  # this makes mocks easier over config.read(filepath)
             if profile_name in config.keys():
                 config_dict = config[profile_name]
-                self.update(config_dict)
             else:
                 profiles = "\n".join(config.keys())
                 msg = (
                     "Couldn't find profile named {} in {} file."
                     " Did you mean one of:\n{}".format(
-                        profile_name, self.filename, profiles
+                        profile_name, cls.filename, profiles
                     )
                 )
                 raise QalxConfigProfileNotFound(msg)
+            return dict(config_dict)
         else:
             raise QalxConfigFileNotFound(
-                "Couldn't find {}".format(self.config_path())
+                "Couldn't find {}".format(cls.config_path())
             )
 
+    def from_inifile(self, profile_name="default"):
+        config_dict = self._load_inifile(profile_name=profile_name)
+        self.update(config_dict)
+
     def __repr__(self):
         return "<%s %s>" % (self.__class__.__name__, dict.__repr__(self))
 
