@@ -1,6 +1,7 @@
 import logging
 import os
 
+import appdirs
 
 TOKEN = None
 """API token"""
@@ -47,3 +48,13 @@ it has not been deleted.
 MAX_RETRY_500 = 2
 """How many times should pyqalx retry the request when receiving a 5XX error
 back from the API?"""
+
+FACTORY_PACK_DIR = appdirs.user_data_dir("qalx", "agiletek")
+r"""
+The directory where factory bot code will be packed and deployed from.
+Defaults to the following:
+
+* Windows: C:\\Users\\<User>\\AppData\\Local\\agiletek\\pyqalx
+* Linux: ~/.local/share/pyqalx on linux
+* Mac: ~/Library/Application Support/pyqalx
+"""
