@@ -32,6 +32,9 @@ class QalxAdapter(object):
     QalxItem() -> Returns a :class:`QalxItem` instance
     """
 
+    # Certain keys shouldn't be attemped to be saved
+    _keys_to_exclude = {"info", "guid", "status"}
+
     def __init__(self, session, *args, **kwargs):
         QalxSession = __import__("pyqalx").QalxSession
         if not isinstance(session, QalxSession):
@@ -128,16 +131,14 @@ class QalxAdapter(object):
         """
         return self.entity_class.entity_type
 
-    @staticmethod
-    def get_keys_to_save(entity):
+    @classmethod
+    def get_keys_to_save(cls, entity):
         """
         When saving an entity not every key should be saved
         """
         # info & guid are both read only
         # status should only be saved via `update_status` (if available)
-        return {
-            k: entity[k] for k in entity if k not in ("info", "guid", "status")
-        }
+        return {k: entity[k] for k in entity if k not in cls._keys_to_exclude}
 
     @staticmethod
     def _validate(**kwargs):
@@ -179,7 +180,7 @@ class QalxAdapter(object):
         response = self._process_api_request(
             "post", self.list_endpoint(**kwargs), **kwargs
         )
-        # TODO: Should we return packable entiies in an unpacked state?
+        # TODO: #3608 Should we return packable entiies in an unpacked state?
         return self.entity_class(response)
 
     def get(self, guid, *args, **kwargs):
@@ -193,7 +194,7 @@ class QalxAdapter(object):
         resp = self._process_api_request("get", endpoint, *args, **kwargs)
         logger.debug(
             f"get {self.entity_class.entity_type} with "
-            "guid {guid} with {endpoint}"
+            f"guid {guid} with {endpoint}"
         )
         return self.entity_class(resp)
 
@@ -758,7 +759,7 @@ class QalxFileAdapter(QalxAdapter):
                     raise QalxFileError(
                         "Invalid file `{0}`".format(input_file)
                     )  # noqa
-            kwargs["file"] = file
+            kwargs[self.entity_class._file_key] = file
         return super(QalxFileAdapter, self)._validate(**kwargs)
 
     def add(self, input_file, file_name, upload=True, **kwargs):
@@ -803,27 +804,27 @@ class QalxFileAdapter(QalxAdapter):
                 # upload the file manually
                 return entity
         # Ensure "put_url" is removed even if we are not changing the file
-        entity["file"].pop("put_url", None)
+        entity[self.entity_class._file_key].pop("put_url", None)
         return entity
 
     def _upload_file(self, entity, input_file):
         # Only upload if everything was OK with the item creation
         # and the user wants to upload the file automatically.
         s3_resp_ok, s3_data = self.session.rest_api._upload_to_s3(
-            entity, input_file
+            entity, input_file, self.entity_class._file_key
         )  # noqa
         if s3_resp_ok is False:
             # The S3 upload failed, return the failed S3 response in place
             # of the response from the api.  Any implementing client will
             # have to handle tidying up any incorrect data.
-            # TODO: Handle errors in a consistent way in adapters
+            # TODO: #3928 Handle errors in a consistent way in adapters
             m = "API request error, message:\n\n-vvv-\n\n"
             m += "\n".join([f"{k}: {v}" for k, v in s3_data.items()])
             m += "\n\n-^^^-"
             raise QalxAPIResponseError(m)
         else:
             # Always pop the `put_url` off if it's a success
-            entity["file"].pop("put_url")
+            entity[self.entity_class._file_key].pop("put_url")
         return entity
 
 
