@@ -5,16 +5,12 @@ import uuid
 
 import boto3
 import dateutil
-import logging
 from botocore.exceptions import ClientError
 
 from pyqalx.core.entities import QalxEntity, Group, Item, Set
 from pyqalx.core.errors import QalxQueueError
 
 
-logger = logging.getLogger(__name__)
-
-
 class QueueResponse:
     """a response from a remote queue"""
 
@@ -27,8 +23,9 @@ class QueueResponse:
         self._raw_response = response
 
 
-class QueueMessage:
-    """a message frm the queue
+class QueueMessage(threading.Thread):
+    """a message frm the queue.  subclasses threading.Thread so that a
+    heartbeat can be started as soon as the message is received from the Queue
 
     attributes:
         body: message body
@@ -36,79 +33,141 @@ class QueueMessage:
         confirm receipt
     """
 
-    def __init__(self, message):
+    def __init__(self, message, worker, visibility, *args, **kwargs):
         """
 
         :param message: the full raw message from the queue
         """
+        super(QueueMessage, self).__init__(*args, **kwargs)
         self._raw_message = message
         self.body = json.loads(message["Body"])
         self.receipt_handle = message["ReceiptHandle"]
+
+        # Set up the heartbeat event and start the thread.
+        # This will "beat" every 50% of visibility seconds and increase
+        # the visibility timeout of the message on the queue.
         self._heartbeats = 1
-        self._heartbeat = None
+        self._heartbeat = threading.Event()
+        self._worker = worker
+        self._logger = self._worker.logger
+        self._queue = self._worker.queue
+        # May not be an int if its come from the config
+        self._visibility = int(visibility)
+        # Start the threaded heartbeat as soon as we can to prevent
+        # message timeouts expiring if it takes a while to start processing them
+        self.start()
 
     def _stop_heartbeat(self):
-        if self._heartbeat is not None:
-            self._heartbeat.set()
-            # Make sure that if we start the heartbeat again on this message
-            # it starts from 1
-            self._heartbeats = 1
+        """
+        Stops any heartbeat that might be mid beat.  At this point the message
+        should be fully processed or the max visibility has been reached.
+        """
+        self._heartbeat.set()
 
-    def _do_heartbeat(self, visibility, queue):
+    def _increase_visibility_timeout(self, visibility_timeout):
+        """
+        Helper for increasing the visibility timeout of this message.  If the
+        timeout is set to > MAX_VISIBILITY_TIMEOUT an error will be logged and
+        the timeout will be set to MAX_VISIBILITY_TIMEOUT
+        :param visibility_timeout: The visibility timeout the message should
+        have. This will set the timeout to be a total number of seconds from
+        when the message first went in flight
+        :type visibility_timeout: int
+        """
+        MAX_VISIBILITY_TIMEOUT = 43200  # 12 hours
+        if visibility_timeout > MAX_VISIBILITY_TIMEOUT:
+            # SQS doesn't recalculate the max timeout if it were to be given
+            # a value greater than the maximum.
+            visibility_timeout = MAX_VISIBILITY_TIMEOUT
+            docs = "https://docs.qalx.io/queues#in-flight-maximum"
+            msg = (
+                f"Max visibility timeout of {MAX_VISIBILITY_TIMEOUT} reached."
+                f"The job will go back onto the queue when "
+                f"this timeout expires. See {docs} for more information. "
+            )
+            self._logger.warning(msg)
+            # Stop the heartbeat, it cannot be increased any further than
+            # MAX_VISIBILITY_TIMEOUT.
+            self._stop_heartbeat()
+
+        try:
+            self._queue.broker_client.change_message_visibility(
+                QueueUrl=self._queue["info"]["queue_url"],
+                ReceiptHandle=self.receipt_handle,
+                VisibilityTimeout=visibility_timeout,
+            )
+        except ClientError as exc:
+            # ClientError: There was an issue changing the visibility.
+            #              This could be a networking issue or the message no
+            #              longer exists on the queue
+            self._logger.debug(
+                f"Received ClientError when extending heartbeat."
+                f"  Error was: `{exc}`"
+            )
+            # Not ideal but the only way to determine from the response
+            # if the message is deleted.
+            message_deleted = (
+                "Message does not exist" in exc.response["Error"]["Message"]
+            )
+
+            if message_deleted:
+                # The message was deleted from the queue.  The heartbeat should
+                # have been stopped but if the user deleted the message in a
+                # non standard way it may not have been.  Therefore, make
+                # sure the heartbeat is stopped to prevent spamming the logs.
+                self._stop_heartbeat()
+            return False
+        return True
+
+    def run(self):
         """
         A QalxJob might take a long time to process.  Therefore, we
         use a heartbeat on the message to keep it in flight for as long
-        as possible.
-        :param visibility: The `MSG_BLACKOUTSECONDS` from the bot_config.
-        :param queue: An instance of :class:~queue.Queue
-        :return:
+        as possible.  This runs in a Thread on QueueMessage instantiation
+        to ensure that the message timeout doesn't expire before processing
+        even begins (i.e. if a batch of messages was returned)
+        As per the SQS docs at:
+
+        https://docs.aws.amazon.com/AWSSimpleQueueService/latest
+        /SQSDeveloperGuide/sqs-visibility-timeout.html
+
+        This will specify the initial visibility timeout (for example, 2 minutes)
+        and then—as long as your consumer still works on
+        the message—keep extending the visibility timeout by 2
+        minutes every minute.
+        This uses `visibility` as the initial visibility timeout so every
+        `visibility / 2` seconds will increase the timeout by
+        `visibility * number of hearbeats` from the initial visibility time.
         """
-        if self._heartbeat is None or self._heartbeat.is_set() is False:
-            # We only do the heartbeat if it hasn't been started yet and we
-            # haven't unset it
-            self._heartbeat = threading.Event()
+        while self._heartbeat.is_set() is False:
+            # We only do the heartbeat if it hasn't been set (meaning the
+            # heartbeat was stopped due to message deletion)
 
             # Ensure it's an int - it might not be if it's come from the config
-            visibility = int(visibility)
+            if self._heartbeats != 1:
+                # No need to change visibility on the first heartbeat as the
+                # message will have the default timeout still.
+                visibility_timeout = self._visibility * self._heartbeats
 
-            visibility = visibility * self._heartbeats
-
-            # The max timeout for a message is 12 hours in seconds
-            MAX_VISIBILITY_TIMEOUT = 43200
-            if visibility <= MAX_VISIBILITY_TIMEOUT:
-                # Don't change visibility the first time as the message will
-                # be in flight anyway
-                try:
-                    queue.broker_client.change_message_visibility(
-                        QueueUrl=queue["info"]["queue_url"],
-                        ReceiptHandle=self.receipt_handle,
-                        VisibilityTimeout=visibility,
-                    )
-                except ClientError as exc:
-                    # ClientError: There was an issue changing the visibility.
-                    #    This can happen if we delete the message at the same
-                    #    time as we are changing the visibility (which is
-                    #    possible as the heartbeat is a thread within the
-                    #    Worker process).  Just return and let the Worker
-                    #    handle the heartbeat again (probably resulting in
-                    #    stopping it)
-                    logger.debug(
-                        f"Got ClientError when extending heartbeat.  "
-                        f"Error was: `{exc}`"
-                    )
+                success = self._increase_visibility_timeout(visibility_timeout)
+                if not success:
+                    # It wasn't successful, so just return without waiting the
+                    # usual amount of time so that the thread can try again.
                     # Wait for a single second to help prevent multiple
                     # requests being fired to AWS
                     self._heartbeat.wait(1)
-                    return
+                    continue
 
-            # We keep track of the number of heartbeats this message has had
-            # and simply increase the visibility timeout exponentially
-            self._heartbeats += 1
-            # Get roughly 90 percent of the updated visibility time.
+            # Get roughly 50 percent of the default visibility time.
             # Sleep until that time expires and then increase the
             # message visibility the next time this method is called
-            ninety_percent = visibility - int(visibility / 10)
-            self._heartbeat.wait(ninety_percent)
+            self._heartbeat.wait(int(self._visibility / 2))
+            self._worker._check_queue_credentials()
+
+            # We keep track of the number of heartbeats this message has had
+            # and increase the timeout by `self._visibility * self._heartbeats`
+            # for each heartbeat (after the first one)
+            self._heartbeats += 1
 
 
 class Queue(QalxEntity):
@@ -217,7 +276,7 @@ class Queue(QalxEntity):
 
         return QueueResponse(responses)
 
-    def get_messages(self, max_num_msg, visibility, waittime):
+    def get_messages(self, max_num_msg, visibility, waittime, worker):
         """
         get messages from the queue
 
@@ -225,6 +284,7 @@ class Queue(QalxEntity):
         :param visibility: time in seconds until the message becomes visible
                            again on the queue
         :param waittime: time to wait for a message
+        :param worker: The ~pyqalx.bot.Worker instance that called this method
         :return: list of `pyqalx.core.entities.queue.QueueMessage`
         """
 
@@ -234,10 +294,12 @@ class Queue(QalxEntity):
             VisibilityTimeout=visibility,
             WaitTimeSeconds=waittime,
         )
-
         if responses.get("Messages"):
             return [
-                QueueMessage(response) for response in responses["Messages"]
+                QueueMessage(
+                    message=response, worker=worker, visibility=visibility
+                )
+                for response in responses["Messages"]
             ]
         return []
 
@@ -255,6 +317,7 @@ class Queue(QalxEntity):
                 QueueUrl=self["info"]["queue_url"],
                 ReceiptHandle=message.receipt_handle,
             )
+            message._stop_heartbeat()
 
     def submit_entity(self, entity):
         """put a qalx entity on the queue
