@@ -83,6 +83,7 @@ def build_default_config(LEVEL, LOG_FILE_PATH):
             "pyqalx.bot": {"handlers": ["filehandler"], "level": LEVEL},
             "pyqalx.config": {"handlers": ["filehandler"], "level": LEVEL},
             "pyqalx.transport": {"handlers": ["filehandler"], "level": LEVEL},
+            "pyqalx.factories": {"handlers": ["filehandler"], "level": LEVEL},
             # Users can do `logging.getLogger('pyqalx.integration') in their
             # pyqalx implementation to keep all pyqalx logs together
             "pyqalx.integration": {"handlers": ["filehandler"], "level": LEVEL},
@@ -156,6 +157,9 @@ def exception_hook(exc_type, exc_value, exc_traceback, extra=None, logger=None):
     (i.e. @bot.process).  This catches all errors and logs them and also
     prints out the traceback
     """
+    if logger is None:
+        logger = get_logger(exc_type, exc_value, exc_traceback)
+
     if issubclass(exc_type, QalxError):
         # Only handle `Qalx*` Errors.  We don't want to consume anything the
         # user has done wrong.  This will however handle issues with decorated
@@ -165,8 +169,6 @@ def exception_hook(exc_type, exc_value, exc_traceback, extra=None, logger=None):
         if issubclass(exc_type, KeyboardInterrupt):
             sys.__excepthook__(exc_type, exc_value, exc_traceback)
             return
-        if logger is None:
-            logger = get_logger(exc_type, exc_value, exc_traceback)
 
         logger.error(
             msg="Qalx Exception:",
@@ -178,6 +180,12 @@ def exception_hook(exc_type, exc_value, exc_traceback, extra=None, logger=None):
             exc_type, exc_value, exc_traceback, chain=False
         )
     else:
+        logger.error(
+            msg="Unhandled Exception:",
+            exc_info=(exc_type, exc_value, exc_traceback),
+            extra=extra,
+        )
+
         # This is an exception elsewhere in the users codebase outside of qalx
         # or any bot function. We don't handle it.  Just reraise it for
         # the user to fix
