@@ -124,20 +124,25 @@ class BasePyQalxAPI(object):
         )
         return url
 
-    def _upload_to_s3(self, data, input_file):
+    def _upload_to_s3(self, data, input_file, file_key):
         """
         Given a response object with a file url in will attempt to
         PUT the file onto S3
 
-        :param data: The data from the api
-        :param input_file: The path to the file to upload or the file itself
+        :param data:        The data from the api
+        :param input_file:  The path to the file to upload or the file itself
+        :param file_key:    The file key in the data dictionary
         :return: response
         """
-        url = data["file"]["put_url"]
-        if not self.is_filestream(input_file):
-            file_data = open(input_file, "rb").read()
+        url = data[file_key]["put_url"]
+
+        if self.is_filestream(input_file):
+            # seek(0) because the file could be at the end if it has already
+            # been uploaded (i.e. if add_many uses the same file for
+            # multiple entities)
+            file_data = input_file.seek(0)
         else:
-            file_data = input_file
+            file_data = open(input_file, "rb").read()
         return self._build_request(
             url=url, method="PUT", include_auth_headers=False, data=file_data
         )
